@@ -10,20 +10,20 @@
 //!
 //! ## Getting started
 //! Most programs that interact with pipewire will need the same few basic objects:
-//! - A [`MainLoop`] that drives the program, reacting to any incoming events and dispatching method calls.
+//! - A [`MainLoop`](`main_loop::MainLoop`) that drives the program, reacting to any incoming events and dispatching method calls.
 //!   Most of a time, the program/thread will sit idle in this loop, waiting on events to occur.
-//! - A [`Context`] that keeps track of any pipewire resources.
-//! - A [`Core`] that is a proxy for the remote pipewire instance, used to send messages to and receive events from the
+//! - A [`Context`](`context::Context`) that keeps track of any pipewire resources.
+//! - A [`Core`](`core::Core`) that is a proxy for the remote pipewire instance, used to send messages to and receive events from the
 //!   remote server.
 //! - Optionally, a [`Registry`](`registry::Registry`) that can be used to manage and track available objects on the server.
 //!
 //! This is how they can be created:
 // ignored because https://gitlab.freedesktop.org/pipewire/pipewire-rs/-/issues/19
 //! ```no_run
-//! use pipewire::{MainLoop, Context};
+//! use pipewire::{main_loop::MainLoop, context::Context};
 //!
 //! fn main() -> Result<(), Box<dyn std::error::Error>> {
-//!     let mainloop = MainLoop::new()?;
+//!     let mainloop = MainLoop::new(None)?;
 //!     let context = Context::new(&mainloop)?;
 //!     let core = context.connect(None)?;
 //!     let registry = core.get_registry()?;
@@ -35,10 +35,10 @@
 //! Now you can start hooking up different kinds of callbacks to the objects to react to events, and call methods
 //! on objects to change the state of the remote.
 //! ```no_run
-//! use pipewire::{MainLoop, Context};
+//! use pipewire::{main_loop::MainLoop, context::Context};
 //!
 //! fn main() -> Result<(), Box<dyn std::error::Error>> {
-//!     let mainloop = MainLoop::new()?;
+//!     let mainloop = MainLoop::new(None)?;
 //!     let context = Context::new(&mainloop)?;
 //!     let core = context.connect(None)?;
 //!     let registry = core.get_registry()?;
@@ -77,14 +77,13 @@
 //! For example, we can call a function on an interval:
 //!
 //! ```no_run
-//! // We also need to include the `Loop` trait for this.
-//! use pipewire::{MainLoop, Loop};
+//! use pipewire::main_loop::MainLoop;
 //! use std::time::Duration;
 //!
 //! fn main() -> Result<(), Box<dyn std::error::Error>> {
-//!     let mainloop = MainLoop::new()?;
+//!     let mainloop = MainLoop::new(None)?;
 //!
-//!     let timer = mainloop.add_timer(|_| println!("Hello"));
+//!     let timer = mainloop.loop_().add_timer(|_| println!("Hello"));
 //!     // Call the first time in half a second, and then in a one second interval.
 //!     timer.update_timer(Some(Duration::from_millis(500)), Some(Duration::from_secs(1))).into_result()?;
 //!
@@ -95,13 +94,13 @@
 //! ```
 //! This program will print out "Hello" every second forever.
 //!
-//! You can also react to IO or Signals using similar methods on the [`Loop`] trait.
+//! Using similar methods, you can also react to IO or Signals, or call a callback whenever the loop is idle.
 //!
 //! ## Multithreading
 //! The pipewire library is not really thread-safe, so pipewire objects do not implement [`Send`](`std::marker::Send`)
 //! or [`Sync`](`std::marker::Sync`).
 //!
-//! However, you can spawn a [`MainLoop`] in another thread and do bidirectional communication using two channels.
+//! However, you can spawn a [`MainLoop`](`main_loop::MainLoop`) in another thread and do bidirectional communication using two channels.
 //!
 //! To send messages to the main thread, we can easily use a [`std::sync::mpsc`].
 //! Because we are stuck in the main loop in the pipewire thread and can't just block on receiving a message,
@@ -109,47 +108,46 @@
 //!
 //! See the [`pipewire::channel`](`crate::channel`) module for details.
 
-use std::ptr;
-
-pub use context::*;
-pub use core_::*;
-pub use error::*;
-pub use loop_::*;
-pub use main_loop::*;
-pub use properties::*;
-pub use pw_sys as sys;
-pub use spa;
-
 pub mod buffer;
 pub mod channel;
+pub mod client;
 pub mod constants;
-mod context;
-mod core_;
-pub mod data;
-mod error;
+pub mod context;
+pub mod core;
+pub mod device;
+pub mod factory;
 pub mod keys;
 pub mod link;
-mod loop_;
-mod main_loop;
+pub mod loop_;
+pub mod main_loop;
 pub mod metadata;
+pub mod module;
 pub mod node;
+pub mod permissions;
 pub mod port;
-mod properties;
+pub mod properties;
 pub mod proxy;
 pub mod registry;
 pub mod stream;
+pub mod thread_loop;
 pub mod types;
+
+mod error;
+pub use error::*;
+
 mod utils;
 
+pub use pw_sys as sys;
+pub use spa;
+
 // Re-export all the traits in a prelude module, so that applications
 // can always "use pipewire::prelude::*" without getting conflicts
 pub mod prelude {
     pub use spa::prelude::*;
-
-    pub use crate::loop_::Loop;
-    pub use crate::stream::ListenerBuilderT;
 }
 
+use std::ptr;
+
 /// Initialize PipeWire
 ///
 /// Initialize the PipeWire system and set up debugging
